Package: BoolNet
Type: Package
Title: Construction, Simulation and Analysis of Boolean Networks
Version: 2.1.9
Date: 2023-10-02
Authors@R: c(person("Christoph", "Müssel", role = "aut"),
             person("Martin", "Hopfensitz", role = "aut"),
             person("Dao", "Zhou", role = "aut"),
             person(c("Hans", "A."), "Kestler", email = "hans.kestler@uni-ulm.de", role = c("aut", "cre")),
			 person("Armin", "Biere", role = "ctb", comment="contributed PicoSAT code"),
			 person(c("Troy", "D."), "Hanson", role = "ctb", comment="contributed uthash macros"))
Imports: igraph (>= 0.6), XML
Description: Functions to reconstruct, generate, and simulate synchronous, asynchronous, probabilistic, and temporal Boolean networks. Provides also functions to analyze and visualize attractors in Boolean networks <doi:10.1093/bioinformatics/btq124>.
License: Artistic-2.0
LazyLoad: yes
ByteCompile: TRUE
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2023-10-02 08:56:07 UTC; julian_schwab
Author: Christoph Müssel [aut],
  Martin Hopfensitz [aut],
  Dao Zhou [aut],
  Hans A. Kestler [aut, cre],
  Armin Biere [ctb] (contributed PicoSAT code),
  Troy D. Hanson [ctb] (contributed uthash macros)
Maintainer: Hans A. Kestler <hans.kestler@uni-ulm.de>
Repository: CRAN
Date/Publication: 2023-10-02 14:30:04 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 15:12:53 UTC; unix
Archs: BoolNet.so.dSYM
